/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "BaseModel.hxx"

#include "Problem.hxx"
#include "Department.hxx"
#include "Variable.hxx"
#include "Constraint.hxx"

#include <cassert>
#include <limits>

#include <iostream>

BaseModel::BaseModel(Problem& problem, bool use_cuts):
    _problem(problem), _use_cuts(use_cuts)
{
}


BaseModel::~BaseModel()
{
    for (size_type i = 0; i < _variables.size(); i++) delete _variables[i];
    for (size_type i = 0; i < _constraints.size(); i++) delete _constraints[i];
}


const Department* BaseModel::department(ID id) const
{
    return _problem.get_department(id);
}


double BaseModel::get_transport(const Department* dep1, const Department* dep2) const
{
    return (_problem.get_transport(dep1, dep2) + _problem.get_transport(dep2, dep1));
}


void BaseModel::init_d_variables(size_type n)
{
    assert(n >= _problem.get_num_orig());

    _d_variables.resize( n );
    for ( size_type i = 0; i < n; ++i ) {
        _d_variables[i].assign( n, NULL );
    }
    for ( size_type i = 0; i < n; ++i ) {
        for ( size_type j = i+1; j < n; ++j ) {
            const Department* dep1 = department(i);
            const Department* dep2 = department(j);
            Variable* var = new Variable(dep1, dep2,
                                         _variables.size(),
                                         "d", Variable::Cont);
            var->set_cost(get_transport(dep1, dep2));
            _d_variables[i][j] = var;
            _d_variables[j][i] = var;
            _variables.push_back(var);
        }
    }
}

void BaseModel::separate(const std::vector<double>& /*x*/,
                         Separator& /*separator*/)
{
}

double BaseModel::get_upper() const
{
    return std::numeric_limits<double>::infinity();
}
