/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "BigM.hxx"

#include "Problem.hxx"
#include "Department.hxx"

#include <algorithm>
#include <vector>

double compute_simple_bigm(Problem& problem)
{
    double bigm = 0;
    for (size_type i = 0; i < problem.get_num_orig(); i++){
        bigm += problem.get_department(i)->get_length();
    }
    return bigm;
}

double compute_improved_bigm(Problem& problem)
{
    size_type n = problem.get_num_orig();
    std::vector<double> leng;
    for (size_type i = 0; i < n; i++){
        leng.push_back( problem.get_department(i)->get_length() );
    }
    std::sort(leng.begin(), leng.end());

    double bigm = 0;
    size_type start = (n+1)/3;

    for (size_type i = start; i < leng.size(); i++){
        bigm += leng[i];
    }

    return bigm;
}
