/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __BNBSOLVER_HXX__
#define __BNBSOLVER_HXX__

#include "Types.hxx"

#include <memory>
#include <vector>

class Problem;
class BetweennessModel;
class MIPSolver;

// our B&B solver

class BnBSolver
{
protected:
    Problem& _problem;
    size_type _nrows;
    bool _use_cuts;

    MIPSolver& _solver;

    std::vector<std::vector<ID>> _best_rows;

public:
    BnBSolver(Problem& problem, size_type nrows, bool use_cuts, MIPSolver& solver);

    BnBSolver(Problem& problem, bool use_cuts, MIPSolver& solver): BnBSolver(problem, 2, use_cuts, solver) {}

    double solve();

    const std::vector<std::vector<ID>> get_solution() const {
        return _best_rows;
    }

protected:
    typedef std::vector<std::vector<size_type> > rows_vec;

    virtual std::unique_ptr<BetweennessModel> create_submodel(const rows_vec& rows, double upper) = 0;

    virtual bool check_valid(const std::vector<double>& lens, double l0max) = 0;
};

#endif
