/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __CPLEXSOLVER_HXX__
#define __CPLEXSOLVER_HXX__

#include <cplex.h>
#include <vector>
#include <limits>

#include "MIPSolver.hxx"
#include "Separator.hxx"
#include "Constraint.hxx"
#include "Types.hxx"

class BaseModel;
class Problem;

// solve the respective model with Cplex

class CplexSolver: public MIPSolver, private Separator
{
private:
    BaseModel* _model;
    const Problem* _problem;

    CPXENVptr _env;
    CPXLPptr _lp;
    CPXFILEptr _fp;

    // callback data
    void* _cbdata;
    int _wherefrom;
    std::vector<double> _x;

    double _lower_bound;


public:
    CplexSolver(double time_limit = std::numeric_limits<double>::infinity());

    ~CplexSolver();

    double solve(BaseModel& model);

    double get_lower_bound() const { return _lower_bound; }

    std::vector<double> get_solution() const;

private:
    void build_lp();

    // the callbacks
    static int cpx_cut_callback(CPXCENVptr env, void* cbdata, int wherefrom, void* cbhandle, int* useraction_p);
    int cut_callback(CPXCENVptr env, void* cbdata, int wherefrom, int* useraction_p);
    static int cpx_cutlazy_callback(CPXCENVptr env, void* cbdata, int wherefrom, void* cbhandle, int* useraction_p);
    int cutlazy_callback(CPXCENVptr env, void* cbdata, int wherefrom, int* useraction_p);

    bool add_if_violated(const coeff_vec& variables, Constraint::Sense sense, double rhs);
};

#endif
