/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "DRFLPAmaralBaseModel.hxx"

#include "Constraint.hxx"
#include "Department.hxx"
#include "Problem.hxx"


DRFLPAmaralBaseModel::DRFLPAmaralBaseModel(double bigm) :
	_bigm(bigm)
{
}

void DRFLPAmaralBaseModel::init_drflp_constraints()
{
    size_type n = _problem.get_num_orig();
    for ( size_type i = 0; i < n; i++ ){
        Constraint* constraint = new Constraint( Constraint::GE, department(i)->get_length()/2);
        constraint->set_coeff( _p_variables[i], 1.0);
        _constraints.push_back(constraint);
    }
    for ( size_type i = 0; i < n; i++ ){
        Constraint* constraint = new Constraint( Constraint::LE, _bigm - department(i)->get_length()/2);
        constraint->set_coeff( _p_variables[i], 1.0 );
        _constraints.push_back(constraint);
    }

    { // symmetry breaking
        size_type min_i = 0;
        size_type min_j = 1;
        double min_val = get_transport(department(0), department(1));
        for ( size_type i = 0; i < n; i++ ){
            for ( size_type j = i+1; j < n; j++ ){
                if ( get_transport(department(i), department(j)) < min_val){
                    min_i = i;
                    min_j = j;
                    min_val = get_transport(department(i), department(j));
                }
            }
        }
        Constraint* constraint = new Constraint( Constraint::LE, 0.0);
        constraint->set_coeff( _p_variables[min_i], 1.0 );
        constraint->set_coeff( _p_variables[min_j], -1.0 );
        _constraints.push_back(constraint);
    }

}

void DRFLPAmaralBaseModel::init_pos_dist_constraints()
{
    size_type n = _problem.get_num_orig();
    for ( size_type i = 0; i < n; i++){
        for ( size_type j = i+1; j < n; j++ ){
            Constraint* constraint = new Constraint( Constraint::GE, 0.0);
            constraint->set_coeff( _d_variables[i][j], 1.0 );
            constraint->set_coeff( _p_variables[j], -1.0 );
            constraint->set_coeff( _p_variables[i], 1.0 );
            _constraints.push_back(constraint);
        }
    }
    for ( size_type i = 0; i < n; i++){
        for ( size_type j = i+1; j < n; j++ ){
            Constraint* constraint = new Constraint( Constraint::GE, 0.0);
            constraint->set_coeff( _d_variables[i][j], 1.0 );
            constraint->set_coeff( _p_variables[j], 1.0 );
            constraint->set_coeff( _p_variables[i], -1.0 );
            _constraints.push_back(constraint);
        }
    }
}
