/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "DRFLPAmaralModel.hxx"

#include "Problem.hxx"
#include "Department.hxx"
#include "Constraint.hxx"
#include "Variable.hxx"

DRFLPAmaralModel::DRFLPAmaralModel(Problem& problem, bool use_cuts, double bigm):
	AmaralModel(problem, use_cuts),
	DRFLPAmaralBaseModel(bigm)
{
}

void DRFLPAmaralModel::init()
{
    init_a_variables();
    init_d_variables(_problem.get_num_orig());
    init_p_variables();
    init_amaral_constraints();
    init_pos_dist_constraints();
    init_drflp_constraints();
    init_pa_constraints();
}

void DRFLPAmaralModel::init_pa_constraints()
{
    size_type n = _problem.get_num_orig();
    for ( size_type i = 0; i < n; i++){
        for (size_type j = 0; j < n; j++){
            if ( i == j ) continue;
            double l_i = department(i)->get_length();
            double l_j = department(j)->get_length();
            Constraint* constraint = new Constraint( Constraint::LE, _bigm - (l_i+l_j)/2.0);
            constraint->set_coeff( _p_variables[i], 1.0 );
            constraint->set_coeff( _p_variables[j], -1.0 );
            constraint->set_coeff( _a_variables[j][i], _bigm );
            _constraints.push_back(constraint);
        }
    }
}
