/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "DRFLPBnBSolver.hxx"

#include "DRFLPBetweennessModel.hxx"

DRFLPBnBSolver::DRFLPBnBSolver(Problem& problem, bool use_cuts, double bigm,
                               MIPSolver& solver):
    BnBSolver(problem, use_cuts, solver), _bigm(bigm)
{}

DRFLPBnBSolver::DRFLPBnBSolver(Problem& problem, size_type nrows, bool use_cuts, double bigm,
                               MIPSolver& solver):
    BnBSolver(problem, nrows, use_cuts, solver), _bigm(bigm)
{}

std::unique_ptr<BetweennessModel> DRFLPBnBSolver::create_submodel(const rows_vec& rows, double upper)
{
    std::unique_ptr<BetweennessModel> m(new DRFLPBetweennessModel(_problem, rows, _use_cuts, _bigm, upper));
    m->init();
    return m;
}

bool DRFLPBnBSolver::check_valid(const std::vector<double>& lens, double /*l0max*/)
{
    if ( _bigm >= 0 && lens[0] - 0.0001  > _bigm ) return false;
    if (lens.back() <  0.0001) return false;
    return true;
}
