/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "DRFLPSAModel.hxx"

#include "Problem.hxx"
#include "Department.hxx"
#include "Constraint.hxx"
#include "Variable.hxx"
#include "Separator.hxx"
#include "InitSeparator.hxx"

#include <iostream>
#include <cstdlib>


DRFLPSAModel::DRFLPSAModel(Problem& problem, bool use_cuts, double bigm):
	AmaralModel(problem, use_cuts),
	DRFLPAmaralBaseModel(bigm),
    SAModelCommon()
{
}

void DRFLPSAModel::init()
{
    init_a_variables();
    init_d_variables(_problem.get_num_orig());
    init_p_variables();
    init_e_variables();
    init_amaral_constraints();
    init_drflp_constraints();
    init_pos_dist_constraints();
    init_sa_constraints();
    init_sa_base_constraints();
    init_e_constraints();
}

void DRFLPSAModel::init_sa_constraints()
{
    size_type n = _problem.get_num_orig();

    for ( size_type i = 0; i < n; i++ ){
        for ( size_type j = 0; j < n; j++){
            if ( i == j ) continue;
            Constraint* constraint = new Constraint( Constraint::LE, 2.0*(_bigm-department(i)->get_length()/2.0-department(j)->get_length()/2.0) );
            constraint->set_coeff( _p_variables[i], 1.0 );
            constraint->set_coeff( _p_variables[j], -1.0 );
            constraint->set_coeff( _d_variables[i][j], 1.0 );
            constraint->set_coeff( _a_variables[i][j], 2.0*(_bigm-department(i)->get_length()/2.0-department(j)->get_length()/2.0) );
            _constraints.push_back(constraint);
        }
    }
}


