/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __GUROBISOLVER_HXX__
#define __GUROBISOLVER_HXX__

#include "MIPSolver.hxx"
#include "Separator.hxx"
#include "Constraint.hxx"
#include "Types.hxx"

#include <gurobi_c++.h>

class BaseModel;
class Problem;

// solve the respective model with Gurobi

class GurobiSolver: public MIPSolver, private GRBCallback, private Separator
{
private:
    BaseModel* _model;
    const Problem* _problem;

    GRBEnv* _env;
    GRBModel* _lp;
    GRBVar* _vars;

    std::vector<double> _x;

    double _lower_bound;

public:
    GurobiSolver(double time_limit = std::numeric_limits<double>::infinity());

    ~GurobiSolver();

    double solve(BaseModel& model);

    double get_lower_bound() const { return _lower_bound; }

    std::vector<double> get_solution() const;

private:
    void build_lp();

    /// Gurobi callback.
    void callback();

    bool add_if_violated(const coeff_vec& vars,
                         Constraint::Sense sense,
                         double rhs);
};

#endif
