/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __PROPBETWEENNESSMODEL_HXX__
#define __PROPBETWEENNESSMODEL_HXX__

#include "BetweennessModel.hxx"

// our model for PROP
/// possible objective types
enum CostType { Cost1=1, Cost2=2, Cost3=3 };

class PROPBetweennessModel: public BetweennessModel
{
private:
    CostType _ctype; ///< objective type

    bool _use_cliques;

public:

    PROPBetweennessModel(Problem& problem,
                         const rows_vec& rows,
                         bool use_cuts,
                         bool use_cliques,
                         CostType ctype,
                         double upper = std::numeric_limits<double>::infinity());

    void init();

    // compute fix costs for cost type 2
    double compute_fixed_costs();

    void separate(const std::vector<double>& x, Separator& separator);

protected:
    void init_prop_constraints();

    /// Initialize costs on x variables for cost type 2.
    void init_cost2();

    void separate_clique_constraints(Separator& separator);
};

#endif
