/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "Problem.hxx"

#include "Department.hxx"
#include "Types.hxx"

#include <iostream>
#include <cassert>
#include <algorithm>

Problem::Problem(size_type num_orig, size_type n_row, std::vector<size_type> n_each_row):
    _num_rows(n_row),
    _num_orig(num_orig)
{
    if (n_row > 0) {
        _num_each_row = n_each_row;
        if ( _num_rows != n_each_row.size() ){
            std::cerr << "Input data does not fit together" << std::endl;
            exit(EXIT_FAILURE);
        }

        _rows.resize( _num_rows );
        size_type idx = 0;
        for ( size_type i = 0; i < n_row; i++ ){
            for ( size_type j = 0; j < n_each_row[i]; j++){
                _rows[i].push_back( idx );
                idx++;
            }
        }
    }
}


Problem::~Problem()
{
    for ( size_type i = 0; i < _departments.size(); ++i )
        delete _departments[i];
}


void Problem::add_department( Department* department )
{
    assert( department != NULL );

    department->set_id(_departments.size());
    department->set_problem(this);
    _departments.push_back(department);
}


void Problem::set_transport( const Department* source, const Department* dest, double flow )
{
    assert( source != NULL && get_department(source->get_id()) == source );
    assert( dest != NULL && get_department(dest->get_id()) == dest );
    assert( flow >= 0 );

    if ( _transports.size() < get_num_orig() ) {
        _transports.resize( get_num_orig() );
    }

    std::vector<double>& trans = _transports[source->get_id()];
    if ( trans.size() < get_num_orig() ) {
        trans.resize( get_num_orig(), 0 );
    }

    trans[dest->get_id()] = flow;
}


double Problem::get_transport( const Department* source, const Department* dest ) const
{
    assert( source != NULL );
    assert( source != NULL && get_department(source->get_id()) == source );
    assert( dest != NULL );
    assert( get_department(dest->get_id()) == dest );

    if ( _transports.size() <= source->get_id() ) return 0;
    else if ( _transports[source->get_id()].size() <= dest->get_id() ) return 0;
    else return _transports[source->get_id()][dest->get_id()];
}
