/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __PROBLEM_HXX__
#define __PROBLEM_HXX__

#include "Types.hxx"

#include <vector>
#include <cassert>

class Department;

// The collection of all departments.
class Problem
{
private:
    std::vector<Department*> _departments;
    size_type _num_rows;
    std::vector<std::vector<double> > _transports;
    size_type _num_orig;

    std::vector<size_type> _num_each_row;
    std::vector<std::vector<size_type> > _rows;

public:
    Problem(size_type n_orig, size_type n_rows, std::vector<size_type> n_each_row);
    ~Problem();

    size_type get_num_rows() const {
        return _num_rows;
    }

    void add_department( Department* department );

    /// Returns the department with a certain id.
    const Department* get_department( ID id ) const {
        assert( id < _departments.size() );
        return _departments[id];
    }

    /// Returns the department with a certain id or NULL.
    Department* get_department_nc( ID id ){
        assert( id < _departments.size() );
        return _departments[id];
    }

    /// Sets the transport between two departments.
    void set_transport( const Department* source, const Department* dest, double flow );

    /// Returns the connectivity between two departments.
    double get_transport( const Department* source, const Department* dest ) const;

    /// Return the number of departments in this problem.
    size_type get_num_orig() const { return _num_orig; }

    std::vector<size_type> get_num_each_row() const {
        return _num_each_row;
    }
    std::vector<std::vector<size_type> > get_rows() const {
        return _rows;
    }


};

#endif
