/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "Reader.hxx"

#include "Department.hxx"
#include "Problem.hxx"

#include <fstream>
#include <sstream>
#include <iostream>
#include <cstdlib>

Reader::Reader(const std::string& transport_fname, bool has_rows):
    _transport_fname(transport_fname), _has_rows(has_rows)
{}


std::unique_ptr<Problem> Reader::read_problem() const
{

    std::ifstream f;
    f.open ( _transport_fname.c_str() );

    if ( f.bad() || f.fail() ){
        std::cerr << "Error: Cannot open file " << _transport_fname << std::endl;
        exit ( 1 );
    }
    size_type n;
    f >> n;
    size_type num_rows = 2;
    if (_has_rows)
        f >> num_rows;

    std::vector<size_type> n_each_row;
    if (_has_rows) {
        for ( size_type i = 0; i < num_rows; i++ ){
            size_type r;
            f >> r;
            n_each_row.push_back(r);
        }
    }


    std::unique_ptr<Problem> problem(_has_rows
                        ? new Problem(n, num_rows, n_each_row)
                        : new Problem(n, 0, n_each_row));

    for ( size_type i = 0; i < n; i++ ){
        Department* mach = new Department();
        double leng;
        f >> leng;
        mach->set_length( leng );
        mach->set_id(i);
        if (_has_rows) {
            size_type idx = 0;
            size_type sum = 0;
            for ( size_type j = 0; j < n_each_row.size(); j++ ){
                if ( i >= sum && i < sum + n_each_row[j] )
                    idx = j;
                sum = sum + n_each_row[j];
            }
            mach->set_row( idx );
        }
        problem->add_department( mach );
    }

    for ( size_type i = 0; i < n; i++ ){
        for ( size_type j = 0; j < n; j++ ){
            size_type t;
            f >> t;
            if ( t > 0 && i < j ) { // costs symmetric
                problem->set_transport( problem->get_department(i), problem->get_department(j) , t );
            }
        }
    }

    f.close();
    return problem;

}


std::vector<std::string> Reader::split( const std::string& line )
{
    std::vector<std::string> tokens;

    std::string::size_type pos = 0;
    std::string::size_type end;

    do {
        end = line.find(';', pos);
        if ( end == std::string::npos ) end = line.size();

        std::string tok = line.substr(pos, end - pos);
        std::string::size_type b = tok.find_first_not_of(" \t");
        std::string::size_type e = tok.find_last_not_of(" \t");

        if ( b != std::string::npos && e != std::string::npos && b <= e )
            tokens.push_back( tok.substr(b,e-b+1) );
        else
            tokens.push_back( "" );

        pos = end + 1;
    } while ( end < line.size() );

    return tokens;
}
