/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "SFDRFLPAmaralBaseModel.hxx"

#include "Constraint.hxx"
#include "Department.hxx"
#include "Problem.hxx"

SFDRFLPAmaralBaseModel::SFDRFLPAmaralBaseModel()
{
}

void SFDRFLPAmaralBaseModel::init_sfdrflp_constraints()
{
    size_type n = _problem.get_num_orig();
    for ( size_type i = 0; i < n; i++){
        for (size_type j = i+1; j < n; j++){
            double l_i = department(i)->get_length();
            double l_j = department(j)->get_length();
            Constraint* constraint = new Constraint( Constraint::GE, (l_i-l_j)/2.0);
            constraint->set_coeff( _d_variables[i][j], 1.0 );
            for ( size_type k = 0; k < n; k++ ){
                if (k == i) continue;
                double l_k = department(k)->get_length();
                constraint->set_coeff( _a_variables[k][i], -l_k );
            }
            for ( size_type k = 0; k < n; k++ ){
                if (k == j) continue;
                double l_k = department(k)->get_length();
                constraint->set_coeff( _a_variables[k][j], l_k );
            }
            _constraints.push_back(constraint);
        }
    }
    for ( size_type i = 0; i < n; i++){
        for (size_type j = i+1; j < n; j++){
            double l_i = department(i)->get_length();
            double l_j = department(j)->get_length();
            Constraint* constraint = new Constraint( Constraint::GE, (l_j-l_i)/2.0);
            constraint->set_coeff( _d_variables[i][j], 1.0 );
            for ( size_type k = 0; k < n; k++ ){
                if (k == i) continue;
                double l_k = department(k)->get_length();
                constraint->set_coeff( _a_variables[k][i], l_k );
            }
            for ( size_type k = 0; k < n; k++ ){
                if (k == j) continue;
                double l_k = department(k)->get_length();
                constraint->set_coeff( _a_variables[k][j], -l_k );
            }
            _constraints.push_back(constraint);
        }
    }
    /*
    // this constraint can be added to the SF models from the literature to improve them
    {
        // symmetry breaking
        Constraint* constraint = new Constraint( Constraint::EQ, 0.0);
        constraint->set_coeff( _a_variables[0][1], 1.0 );
        _constraints.push_back(constraint);
    }
    */
}
