/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "SFDRFLPBnBSolver.hxx"

#include "PROPBetweennessModel.hxx"

SFDRFLPBnBSolver::SFDRFLPBnBSolver(Problem& problem, bool use_cuts, bool use_reduce,
                                   MIPSolver& solver):
    BnBSolver(problem, use_cuts, solver), _use_reduce(use_reduce)
{}

SFDRFLPBnBSolver::SFDRFLPBnBSolver(Problem& problem, size_type nrows, bool use_cuts, bool use_reduce,
                                   MIPSolver& solver):
    BnBSolver(problem, nrows, use_cuts, solver), _use_reduce(use_reduce)
{}

std::unique_ptr<BetweennessModel> SFDRFLPBnBSolver::create_submodel(const rows_vec& rows, double upper)
{
    std::unique_ptr<BetweennessModel> m(new PROPBetweennessModel(_problem, rows, _use_cuts, false, Cost1, upper));
    m->init();
    return m;
}

bool SFDRFLPBnBSolver::check_valid(const std::vector<double>& lens, double l0max)
{
    if ( _use_reduce &&  lens[0] - l0max - lens[1] > 0.001 )
        return false;
    else
        return true;
}
