/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __SEPARATOR_HXX__
#define __SEPARATOR_HXX__

#include "Constraint.hxx"

#include <vector>

// Callback for separating constraints.
class Separator
{
public:
    // vector of coefficients in a constraint
    typedef std::vector<std::pair<const Variable*, double> > coeff_vec;

    virtual ~Separator();

    /**
     * Adds a new constraint if it is violated.
     *
     * @param vars the variable and coefficients
     * @param sense the sense of the new constraint
     * @param rhs the right-hand side of the new constraint
     *
     * @return true iff the constraint has been added.
     */
    // implemented for each solver variant
    virtual bool add_if_violated(const coeff_vec& vars,
                                 Constraint::Sense sense,
                                 double rhs) = 0;
};

#endif
