/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#include "Variable.hxx"

#include "Department.hxx"

#include <sstream>
#include <cassert>

Variable::Variable( const Department* dep1,
                    size_type index,
                    const std::string& nameprefix,
                    Type type, double lb, double ub, double cost)
    :_index(index),
     _dep1(dep1), _dep2(0), _dep3(0),
     _type(type), _lb(lb), _ub(ub), _cost(cost)
{
    assert(dep1 != 0);

    if (type == Binary) {
        _lb = 0;
        _ub = 1;
    }

    std::ostringstream n;
    n << nameprefix << "_" << _dep1->get_id();
    _name = n.str();
}

Variable::Variable( const Department* dep1, const Department* dep2,
                    size_type index,
                    const std::string& nameprefix,
                    Type type, double lb, double ub, double cost)
    :_index(index),
     _dep1(dep1), _dep2(dep2), _dep3(0),
     _type(type), _lb(lb), _ub(ub), _cost(cost)
{
    assert(dep1 != 0);

    if (type == Binary) {
        _lb = 0;
        _ub = 1;
    }

    std::ostringstream n;
    n << nameprefix
      << "_" << _dep1->get_id()
      << "_" << _dep2->get_id();
    _name = n.str();
}

Variable::Variable( const Department* dep1, const Department* dep2, const Department* dep3,
                    size_type index,
                    const std::string& nameprefix,
                    Type type, double lb, double ub, double cost)
    :_index(index),
     _dep1(dep1), _dep2(dep2), _dep3(dep3),
     _type(type), _lb(lb), _ub(ub), _cost(cost)
{
    assert(dep1 != 0);

    if (type == Binary) {
        _lb = 0;
        _ub = 1;
    }

    std::ostringstream n;
    n << nameprefix
      << "_" << _dep1->get_id()
      << "_" << _dep2->get_id()
      << "_" << _dep3->get_id();
    _name = n.str();
}
