/*
 * This program is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see  <http://www.gnu.org/licenses/>
 */

#ifndef __VARIABLE_HXX__
#define __VARIABLE_HXX__

#include "Types.hxx"

#include <string>

class Department;

// A variable.
class Variable
{
public:
    enum Type { Binary, Cont };

private:
    size_type _index;

    const Department* _dep1;
    const Department* _dep2;
    const Department* _dep3;

    Type _type;
    double _lb;
    double _ub;
    double _cost;

    std::string _name;

public:
    Type get_type() const { return _type; }

    double get_lb() const { return _lb; }
    void set_lb(double lb) { _lb = lb; }

    double get_ub() const { return _ub; }
    void set_ub(double ub) { _ub = ub; }

    double get_cost() const { return _cost; }
    void set_cost(double cost) { _cost = cost; }

    size_type get_index() const { return _index; }

    const Department* get_dep1() const {
        return _dep1;
    }
    const Department* get_dep2() const {
        return _dep2;
    }
    const Department* get_dep3() const {
        return _dep3;
    }

    const std::string& get_name() const { return _name; }
    void set_name(const std::string& name) { _name = name; }

    // variable with one department
    Variable( const Department* dep1,
              size_type index,
              const std::string& nameprefix,
              Type type = Cont,
              double lb = 0,
              double ub = std::numeric_limits<double>::infinity(),
              double cost = 0);

    // variable with two departments
    Variable( const Department* dep1, const Department* dep2,
              size_type index,
              const std::string& nameprefix,
              Type type = Cont,
              double lb = 0,
              double ub = std::numeric_limits<double>::infinity(),
              double cost = 0);

    // variable with three departments
    Variable( const Department* dep1, const Department* dep2, const Department* dep3,
              size_type index,
              const std::string& nameprefix,
              Type type = Cont,
              double lb = 0,
              double ub = std::numeric_limits<double>::infinity(),
              double cost = 0);
};

#endif
